/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import at.ac.tuwien.vitalab.cosine.webservice.ASensorWebService;
import at.ac.tuwien.vitalab.middleware.Middleware;
import at.ac.tuwien.vitalab.middleware.discovery.AServiceQuery;
import at.ac.tuwien.vitalab.middleware.discovery.IServiceSubscriber;
import at.ac.tuwien.vitalab.middleware.discovery.IServiceUpdateReceiver;
import at.ac.tuwien.vitalab.middleware.discovery.MissingServiceAttributeException;
import at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo;

/**
 * Tests whether the CIMS works and lists all found sensors in the network.
 * 
 * @author Lukasz Juszczyk
 *
 */
public class TestCMS {

	/**
	 * Starts the middleware.
	 * 
	 * @param args Arguments are ignored.
	 */
	public static void main(String args[]) {
		
		try {
			
			Middleware.start();
			
			final CMS cims=new CMS();
			
			cims.deploy();
			
			SensorQuery query=new TestCMS().new SensorQuery();
			
			Middleware.getServiceDatabase().subscribe(query, query);
			
			// run  until 'exit' is typed into stdin
			BufferedReader reader=
				new BufferedReader(new InputStreamReader(System.in));
			
			String inputLine=null;
			while ((inputLine=reader.readLine())!=null && 
					!"exit".equals(inputLine.toLowerCase())) {
			}
			
			Middleware.getServiceDatabase().unsubscribe(query);
			
			cims.shutdown();
			
			Middleware.stop();
			
			System.exit(0);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
	}

	/**
	 * Query matching all sensors in the network.
	 * 
	 * @author Lukasz Juszczyk
	 *
	 */
	private class SensorQuery extends AServiceQuery 
	implements IServiceSubscriber, IServiceUpdateReceiver {

		/**
		 * Constructor.
		 *
		 */
		public SensorQuery() {
		}
		
		/* (non-Javadoc)
		 * @see at.ac.tuwien.vitalab.middleware.discovery.AServiceQuery#matches(at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo)
		 */
		protected boolean matches(ServiceInfo info) {
			
			// match all sensors which support subscription
			return (ASensorWebService.WSATTR_VALUE_SENSOR.equals(
					info.getAttribute(ASensorWebService.WSATTR_NAME_SENSOR)) &&
					"supported".equalsIgnoreCase(info.getAttribute(
							"subscription")));
		}

		/* (non-Javadoc)
		 * @see at.ac.tuwien.vitalab.middleware.discovery.IServiceSubscriber#notification(at.ac.tuwien.vitalab.middleware.discovery.AServiceQuery, at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo)
		 */
		public synchronized void notification(
				AServiceQuery query, ServiceInfo info) {
			System.out.println("New sensor:");
			try {
				System.out.println("location= "+info.getUrl());
			} catch (MissingServiceAttributeException e1) {
			}
			try {
				System.out.println("type= "+info.getId());
			} catch (MissingServiceAttributeException e) {
			}
			System.out.println();
			info.subscribeToAvailabilityUpdates(this);
		}

		/* (non-Javadoc)
		 * @see at.ac.tuwien.vitalab.middleware.discovery.IServiceUpdateReceiver#serviceAvailable(at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo)
		 */
		public synchronized void serviceAvailable(ServiceInfo info) {
			System.out.println("Reappeared sensor:");
			try {
				System.out.println("location= "+info.getUrl());
			} catch (MissingServiceAttributeException e1) {
			}
			try {
				System.out.println("type= "+info.getId());
			} catch (MissingServiceAttributeException e) {
			}
			System.out.println();
			
		}

		/* (non-Javadoc)
		 * @see at.ac.tuwien.vitalab.middleware.discovery.IServiceUpdateReceiver#serviceUnavailable(at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo)
		 */
		public synchronized void serviceUnavailable(ServiceInfo info) {
			System.out.println("Disappeared sensor:");
			try {
				System.out.println("location= "+info.getUrl());
			} catch (MissingServiceAttributeException e1) {
			}
			try {
				System.out.println("type= "+info.getId());
			} catch (MissingServiceAttributeException e) {
			}
			System.out.println();
		}
	}
}
