/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.contextrouting;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Vector;

public class ContextRoutingPathReconstructor {

	private LinkedList resultPath = new LinkedList();
	private String lookingFor = "g";
	private LinkedList currentPath = new LinkedList();
	private LinkedList uniquePath = new LinkedList();
	private boolean stillUnique = true;
	private HashMap relations = new HashMap();
	
	
	public void reset() {
		currentPath = new LinkedList();
		resultPath = new LinkedList();
		stillUnique = true;			
		uniquePath = new LinkedList();
	}
	
	private String searchPath(String node) {
		String result;
		if (node.equals(lookingFor)) {
			currentPath.add(node);	
			
			if (resultPath.size() == 0)
				resultPath.addAll(currentPath);
			else { 
				resultPath.clear();
				return "break";	
			}
			
			return "restart";
		}
		
		currentPath.add(node);
		if (stillUnique && uniquePath.isEmpty())
			uniquePath.add(node);
		
		Vector next = (Vector)relations.get(node);
		if (next == null || next.isEmpty()) {
			//reached top with no result -> uniquePath is of no value
			if (resultPath.size() == 0)
				uniquePath.clear();
			
			return "restart";
		}
		result = "";
		for (int i=0; i < next.size(); i++) {
			if (stillUnique && next.size() == 1) {
				uniquePath.add(next.get(0));
			} else
				stillUnique = false;
				
			String nextNode = (String)next.get(i);
			result = searchPath(nextNode);
			if (result.equals("break"))
				break;

			if (result.equals("restart")) {
				currentPath.remove(nextNode);				
			}
				

		}
		return result;
		
	}
	
	public void setRelations(HashMap relations) {
		this.relations = relations;
	}
	
	public LinkedList recreatePath(String bottom, String top) {
		
		if (relations == null || relations.isEmpty())
			return null; //error
		
		reset();
		this.lookingFor = top;
		searchPath(bottom);
//for partial reconstruction -> enable if greenlighted
//		if (resultPath.isEmpty())
//			resultPath = uniquePath;
		
		return resultPath;
	}
	
	//********************************** below here test stuff only ********************************************//
	public void startTheStuff(String node) {
		for (int i=0; i < testArray.length; i++) {
			reset();
			System.out.println("TEST:\t"+testArray[i][0]+" -> "+testArray[i][1]+":");			
			this.lookingFor = testArray[i][1];
			this.searchPath(testArray[i][0]);


			System.out.println("resultVector");
			for (int j=0; j<resultPath.size(); j++) {
				System.out.print(resultPath.get(j)+" ,");
			}
			if (resultPath.size() > 0)
				uniquePath.clear();
			System.out.println("\nlastUniqueNode");
			for (int j=0; j<uniquePath.size(); j++) {
				System.out.print(uniquePath.get(j)+" ,");
			}			
			System.out.println("\n");			
		}
		this.lookingFor = "g";
		this.searchPath("b");
		System.err.println();
	}
	
	public void setupVector() {
		Vector children = new Vector();
		children.add("b");
		children.add("c");
		relations.put("a", children);
		children = new Vector();
		children.add("d");
		relations.put("b", children);
		children = new Vector();
//		children.add("d");
		children.add("e");
		relations.put("c", children);		
		children = new Vector();
		children.add("g");
		children.add("f");
		relations.put("d", children);		
		children = new Vector();
		children.add("g");
		relations.put("e", children);		
		children = new Vector();
//		children.add("g");
		children.add("e");
		relations.put("f", children);		
		children = new Vector();
		relations.put("g", children);				
	}
	
	String[][] testArray = new String[][]
	                                    { 
											{"a","g"},
											{"a","f"},
											{"a","d"},
											{"a","b"},
											{"a","c"},
											{"a","e"},
											
											{"b","d"},
											{"b","f"},
											{"b","g"},
											
											{"c","e"},
											{"c","d"},
											{"c","f"},
											{"c","g"},
											
											{"e","g"},

											{"d","f"},
											{"d","g"},

											{"f","g"}
											
	                                    };

	
	public static void main(String args[]) {
		ContextRoutingPathReconstructor tc = new ContextRoutingPathReconstructor();
		tc.setupVector();
		tc.startTheStuff("a");
	}

}

