/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.contextrouting;

import java.util.Hashtable;

import at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo;

import de.fzi.XPath.Expr;

/**
 * @author hpsaier
 * represents a node on the xpath
 */


public class ContextRoutingXPathNode {

	private String name;
	private String realtionToPrev;
	private String realtionToNext;
	private ContextRoutingXPathNode upToNext;
	private ContextRoutingXPathNode downToNext;
	private String namespace;
	private Expr[] attribute;
	private String unrecognizedAttributes;
	/**
	 * xpath statement working for this nodes result
	 */
	private String xpath;	
	/**
	 * present result-set for this node
	 * [uuid <-> {@link ContextRoutingServiceInfo}]
	 */
	private ServiceInfo[] nodeResult;

	public String getXpath() {
		return xpath;
	}

	public void setXpath(String xpath) {
		this.xpath = xpath;
	}

	protected ContextRoutingXPathNode() {
		
	}
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getRealtionToNext() {
		return realtionToNext;
	}

	public void setRealtionToNext(String realtionToNext) {
		this.realtionToNext = realtionToNext;
	}

	public ContextRoutingXPathNode getUpToNext() {
		return upToNext;
	}

	public void setUpToNext(ContextRoutingXPathNode upToNext) {
		this.upToNext = upToNext;
	}

	public Expr[] getAttribute() {
		return attribute;
	}

	public void setAttribute(Expr[] attribute) {
		//MUST NOT be null!!!
		if (attribute == null)
			attribute = new Expr[]{};
		
		this.attribute = attribute;
	}

	public ContextRoutingXPathNode getDownToNext() {
		return downToNext;
	}

	public void setDownToNext(ContextRoutingXPathNode downToNext) {
		this.downToNext = downToNext;
	}

	public String getNamespace() {
		return namespace;
	}

	public void setNamespace(String namespace) {
		if (namespace == null)
			namespace = "";
		this.namespace = namespace;
	}

	public String getRealtionToPrev() {
		return realtionToPrev;
	}

	public void setRealtionToPrev(String realtionToPrev) {
		this.realtionToPrev = realtionToPrev;
	}

	public ServiceInfo[] getNodeResult() {
		return nodeResult;
	}

	public void setNodeResult(ServiceInfo[] serviceInfos) {
		this.nodeResult = serviceInfos;
	}
/*
	public boolean isDummy() {
		return isDummy;
	}

	public void setDummy(boolean isDummy) {
		this.isDummy = isDummy;
	}
	*/

	public String getUnrecognizedAttributes() {
		return unrecognizedAttributes;
	}

	public void setUnrecognizedAttributes(String unrecognizedAttributes) {

		if (unrecognizedAttributes == null)
			unrecognizedAttributes = "";
		this.unrecognizedAttributes = unrecognizedAttributes;
	}
}
