/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.contextrouting;

import java.io.Reader;
import java.io.StringReader;


import de.fzi.XPath.Expr;
import de.fzi.XPath.NameTest;
import de.fzi.XPath.Step;
//import de.fzi.XPath.Parser.ParseException;
import de.fzi.XPath.Parser.Token;
import de.fzi.XPath.Parser.XPathParser;
import de.fzi.XPath.Parser.XPathParserConstants;
import edu.wustl.mobilab.sliver.log.Logger;
import edu.wustl.mobilab.sliver.log.LoggerFactory;

public class ContextRoutingXPathParser {
	
	/**
	 * logger
	 */
	private static Logger logger=
		LoggerFactory.getLogger(ContextRoutingXPathParser.class);

	/**
	 * stream version of the xpath statement
	 */
	private Reader stream;
	/**
	 * parser object
	 */
	private XPathParser pathParser;
	/**
	 * points to the last node in the tree
	 */
	private ContextRoutingXPathNode lastNode;
	/**
	 * points to the first node in the tree
	 */
	private ContextRoutingXPathNode firstNode;
	/**
	 * the statement
	 */
	private String XPath;
	
	public static final String GOOD = "GOOD";
	
//	private String shortenedXPath;
	/**
	 * @param xpath the statement to parse
	 */
	protected ContextRoutingXPathParser(String xpath) throws Exception{
		XPath = xpath;
		stream = new StringReader(xpath);
		try {
			pathParser = new XPathParser(stream);
		} catch (Exception e) {
			throw new Exception("Could not create parser\n"+e.getMessage());
			//logger.error("Could not create parser\n"+e.getMessage());
		}
		//minimize debug output
		pathParser.disable_tracing();
		lastNode = null;
	}
	
	/**
	 * parses only nodes, nodes relation, and predicates and stores to 
	 * a tree of ContextRoutingXPathNode
	 * @return parse status
	 */
	public String parseXPathStatement()/* throws Exception*/{
		//boolean result = false;
		String result = ContextRoutingXPathParser.GOOD;
		ContextRoutingXPathNode prevNode = null, newNode = null;
		Token token = null;
		do {
			//move forward
			try {
				token = pathParser.getNextToken();
/*			} catch (de.fzi.XPath.Parser.TokenMgrError e){
				throw new Exception ("When analyzing token of XPath encountered following error: "+e.getMessage());
			}*/
			} catch (Throwable e){
				return "PARSER-ERROR: When analyzing token of XPath encountered following error: "+e.getMessage();
			}				
			//parsed to the end
			if (pathParser.token.kind == XPathParserConstants.EOF)
				break;
			
			String tokenName = token.toString();

			if (tokenName.equals("//") || tokenName.equals("/")) {
				newNode = new ContextRoutingXPathNode();
				//set firstNode only once!
				if (firstNode == null) {
					firstNode = newNode;					
				}
				else { //set relation from prev to actual
					prevNode.setRealtionToNext(tokenName);
				}
				//set relation to next 
				newNode.setRealtionToPrev(tokenName);
				//clear realtion to next
				newNode.setRealtionToNext("");
				
				//connect to eachother
				newNode.setUpToNext(prevNode);
				if (prevNode != null)
					prevNode.setDownToNext(newNode);
				newNode.setUnrecognizedAttributes("");
			}
			
			Step s = null;
			try {
				s = pathParser.Step();
//			}	catch (de.fzi.XPath.Parser.TokenMgrError e){
			} catch (Throwable e){

				return "PARSER-ERROR: for xpath:'"+this.XPath+"'\n"+e.getMessage();
			}
			predicateParser(s, newNode);
			//NameTest for correct but unrecognized path
			if (s.getNodeTest() instanceof NameTest) {
				NameTest nameTest = (NameTest) s.getNodeTest();
				if (nameTest.getNameTestType() == NameTest.ANY || 
					nameTest.getNameTestType() == NameTest.PREFIX_ANY ||
					nameTest.getNameTestType() == NameTest.PREFIX_NAME) {
					//reset relations
					newNode = newNode.getUpToNext();
					newNode.setDownToNext(null);
					break;	
				}
				//as expected
				else if(nameTest.getNameTestType() == NameTest.NAME) {
					if (newNode != null) 	
						newNode.setName(s.getNodeTest().toString());
					prevNode = newNode;
				}
				//exception
				else if (nameTest.getNameTestType() == NameTest.INVALID) {
					return "PARSER-ERROR: XPath statement contains invalid node";
					//TODO
					//break;
				}

			} else {
				//logger.warning("Could not recognize node of XPath! Stopping parsing action...");
				return "PARSER-ERROR: Could not recognize node of XPath! Stopping parsing action...";
			}
					
		} while (true);
		
		//fetch the last node for later search
		lastNode = newNode;
	
		return result;
	}
	/**
	 * check if node is of type ANY
	 * @param node 
	 * @return
	 */
	public static boolean isAnyNode(String nodeName) {
		return ((new NameTest(nodeName).getNameTestType()) == NameTest.ANY);
	}
	private void predicateParser(Step s, ContextRoutingXPathNode newNode) {
		Expr[] attributes = new Expr[s.getPredicatesCount()];
		
		for (int i= 0; i < s.getPredicatesCount(); i++) {
				attributes[i] = (Expr)s.getPredicate(i);
			}
		
		newNode.setAttribute(attributes);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
	}

	public ContextRoutingXPathNode getLastNode() {
		return lastNode;
	}

	public String getXPath() {
		return XPath;
	}

	public void setXPath(String path) {
		XPath = path;
	}

	public ContextRoutingXPathNode getFirstNode() {
		return firstNode;
	}

//	public String getShortenedXPath() {
//		return shortenedXPath;
//	}
//
//	public void setShortenedXPath(String shortenedXPath) {
//		this.shortenedXPath = shortenedXPath;
//	}
}
