/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.contextrouting;

import java.util.Hashtable;
import java.util.Iterator;

import at.ac.tuwien.vitalab.cosine.rating.ARating;
import at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo;

/**
 * @author hpsaier
 *	return individual rating for each ServiceInfo
 */
public class IndividualContextRating extends IContextRating{

	
	/**
	 * rating table of structure [ServiceInfo <->[ARating <-> raring value]]
	 */
	private static Hashtable ratingTable = new Hashtable(5);
	private static double bestRating = 0;	

	/* (non-Javadoc)
	 * @see at.ac.tuwien.vitalab.cosine.contextrouting.IContextRating#getRating(at.ac.tuwien.vitalab.cosine.rating.ARating, at.ac.tuwien.vitalab.cosine.contextrouting.ServiceInfo)
	 */
	public double getRating(ARating type, ServiceInfo info) {
		// TODO Auto-generated method stub
		double result = 0;
		Hashtable type_value = (Hashtable) ratingTable.get(info);
		if (type_value == null)
			return bestRating;
		
		result = ((Double) type_value.get(type)).doubleValue();
		
		return result;
	}

	public double getRating(ServiceInfo info) {
		// TODO Auto-generated method stub
		double result = 0;
		
		Hashtable type_value = (Hashtable) ratingTable.get(info);
		
		if (type_value == null) {
			return bestRating; //if never been rated -> return best
		}
		
		for (Iterator it=type_value.values().iterator(); it.hasNext(); ) {
			result+=(double)((Double)it.next()).doubleValue(); 
		}
		return result;	
	}

	public void setRating(ARating type, ServiceInfo serviceInfo, double value) {
		// TODO Auto-generated method stub
		Hashtable type_value = (Hashtable) ratingTable.get(serviceInfo);
		if (type_value == null)
			type_value = new Hashtable(3);
		value = prepareValue(type, value);
		type_value.put(type, new Double(value));
		ratingTable.put(serviceInfo, type_value);
	}

	private double prepareValue(ARating type, double value) {
		double result;
		double max = type.getMaxRating();
		double min = type.getMinRating();
		double weight = type.getWeightRating();

		if (weight == 0)
			weight = 1;

		//no max and min set
		if ((max+min) == 0) {
			result = value * weight;			
		}
		else {
			result = ((value - min)/(max - min)) * weight;
//			System.err.println("(("+value+" - "+min+")/("+max+" - "+min+")) * "+weight+" = "+result);
		}
		if (bestRating < result)
			bestRating = result+1;
		
		return result;
	}

	public int getNumberOfRatings(ServiceInfo info) {
		// TODO Auto-generated method stub
		Hashtable type_value = (Hashtable) ratingTable.get(info);
		if (type_value == null)
			return 0;
		else
			return type_value.size();
		
	}
	
}
