/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.initservice;

import java.awt.Color;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.xmlpull.v1.XmlPullParserException;

import edu.wustl.mobilab.sliver.log.Logger;
import edu.wustl.mobilab.sliver.log.LoggerFactory;

import at.ac.tuwien.vitalab.cosine.ContextData;
import at.ac.tuwien.vitalab.cosine.Utils;
import at.ac.tuwien.vitalab.cosine.XMLData;
import at.ac.tuwien.vitalab.cosine.contextrouting.ContextRoutingService;
import at.ac.tuwien.vitalab.cosine.contextrouting.IContextRoutingServiceCallback;
//import at.ac.tuwien.vitalab.cosine.contextrouting.ServiceInfo;
import at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo;

import thinlet.Thinlet;

/**
 * @author hpsaier
 * call order
 * .) create a  MobileDeviceInitUI object
 * .) generate the fields you want to display
 * .) try to gather access (enter critical section)
 * .) add generated fields to UI
 * .) call object using FrameLauncher
 * .) wait for result (blocking)
 * .) dispose your FrameLauncher
 * 
 */
public class MobileDeviceInitUI extends Thinlet implements  IContextRoutingServiceCallback{

	/**
	 * WTF?
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * map containing all user input fields in presentation order
	 * layout: [key -> [label, ui_element]]
	 */
	private Map uiMap = new LinkedHashMap();
	/**
	 * logger
	 */
	private static Logger logger=
		LoggerFactory.getLogger(MobileDeviceInitService.class);

	/**
	 * layout [idSubscriber -> [objectkey, value1, value2,...]]
	 */
	private Map idSubscriptionMap = new LinkedHashMap();
	/**
	 * Iterator that creates the UI;
	 */
	private Iterator uiIt = null;
	/**
	 * UI object ground panel. ALL objects are add to it.
	 */
	private Object mainPanel = null;
	/**
	 * default number of input fields per page
	 */
	private int fieldsPerPage = 4;

	/**
	 * to sync return of result
	 */
	private static int tokenResult = 0;
	/**
	 * to sync GUI display
	 */
	private static int tokenUI = 1;
	
	/**
	 * uniquely defines the fields requiring user input
	 */
	public static final String REQUIRED_MARKER = "*";
	
	public static final String DONE = "Done";
	public static final String NEXT = "Next >>";
	public static final String PREV = "<< Prev";
	
	/**
	 * current button(set) [label, button]
	 */
	public Object[] buttonset;


	/**
	 * Constructor no params
	 */
	public MobileDeviceInitUI() {
		this.initMainWindow();
	}

//	protected void handelException(Exception e) {
//		System.err.;
//	}
	/**
	 * initializes general panel layout and shows ui
	 */	
	private void initMainWindow() {

		// setup the main features of mainPanel
		mainPanel = create("panel");
		setInteger(mainPanel, "columns", 2);
		setInteger(mainPanel, "top", 4);
		setInteger(mainPanel, "left", 4);
		setInteger(mainPanel, "right", 4);
		setInteger(mainPanel, "gap", 4);

		// ...and add to the empty UI		
		add(mainPanel);

	}
	
/*	public void paint(Graphics g) {
		try {
			super.paint(g);
		}catch (Exception e){
		}
	}
*/	
	/* (non-Javadoc)
	 * @see at.ac.tuwien.vitalab.cosine.contextrouting.IContextRoutingServiceCallback#queryUpdate(java.lang.Object[], java.lang.String)
	 */	
	public void queryUpdate(Object[] resultArray, String idSubscription){
		Object[] object = null;
		Object[] key_uuid = (Object[])idSubscriptionMap.get(idSubscription);
		XMLData cd = null;
		
	
		String key = (String)key_uuid[0];
		if (key != null)
			object = (Object[])uiMap.get(key);

		if (object == null || object.length < 2)
			return;

		Object component = object[1];
		Object selected = null;
//		try {		
		if(isOfType(component, "combobox")) {
			selected = getSelectedItem(component);			
			if (selected == null) {
				//create dummy of ini data -> so is selected when found
				selected = create("choice");
				setString(selected,"name", (String)key_uuid[1]);
//				Object button = getComponent(component, )
			}
			//remove all choice
			removeAll(component);
			
		}
//		} catch (Exception e){
//			System.err.println("********************* caught in the act!!!");
//		}
		for (int i=0; resultArray.length > i; i++) {			
			if(resultArray[i] instanceof ServiceInfo[]) {
				ServiceInfo[] info = (ServiceInfo[]) resultArray[i];
				if (info != null) {
					for (int j = 0; j < info.length; j++) {						
						if(isOfType(component, "combobox")) {
							String context = info[j].getAttribute(ContextData.IDNAME);
							try {
								// Create a ContextData object
								cd = new XMLData(context);
							} catch (XmlPullParserException e) {
								logger.error("XMLData gather problem: "+e.getMessage());
							} catch (IOException e) {
								logger.error("XMLData gather problem: "+e.getMessage());
							}
							
							String identifier = formLabelFromAttributes(cd);
							//Hashtable ht = info[j].ge;
							String uuid = cd.getAttributeByName("uuid");								
							
							this.createChoice(component, uuid, identifier);
							
						}
					}
					//reselect the component of choice 
					setInteger(component, "selected", getSelectedToObject(component,selected));
				}
			}
		}	
	}
	
	/** get all attributes except some in a String
	 * @param serviceInfo serving the attributes
	 * @return String used as label 
	 */
	private String formLabelFromAttributes(XMLData contextData) {
		String result = "";
		Hashtable ht = contextData.getAttributes();
//		List sortedList = new ArrayList(ht.keySet());
		for (Iterator it = ht.keySet().iterator(); it.hasNext(); ) {
			String key = (String) it.next();
			if (key.equals("uuid") || key.equals("belongsTo"))
				continue;
			result += (String) ht.get(key)+"_"; 			
		}
						
	
		if (result.length() > 1 )
			result = result.substring(0, result.length()-1);
		return result;
	}

	public void addFieldsToUI() {
		uiIt = uiMap.keySet().iterator();
		this.addRemainingFields();

	}
	
	private void startOverFields() {
		addFieldsToUI(); 
	}
	/**
	 * adds the remaining (uiIt) fields to the mainPanel
	 */
	private void addRemainingFields() {
		int maxShowFields = this.fieldsPerPage;
		Object[] uiObject;

		//clear up main panel
		removeAll(mainPanel);
		//... add new fields
		while (maxShowFields > 0) {
			if (uiIt.hasNext()) {
				uiObject = (Object[]) uiMap.get(uiIt.next());
				add(mainPanel, uiObject[0]);
				add(mainPanel, uiObject[1]);
			}
			maxShowFields--;
		}
		//... finally add buttons 
		if (uiIt.hasNext()) {
//			uiObject = this.createStdButtons();
			uiObject = this.createNextButton();
			this.buttonset = uiObject;
			add(mainPanel, uiObject[0]);
			add(mainPanel, uiObject[1]);
		}
		else {
			uiObject = this.createDoneButton();
			this.buttonset = uiObject;			
			add(mainPanel, uiObject[0]);
			add(mainPanel, uiObject[1]);			
		}
			
	}
	
	/**
	 * @param key:
	 *            defines the type of the input field and the id e.g. text_Name
	 * @param value:
	 *            value of the field
	 */
	public void generateFields(String key, String value, boolean isOptional, 
								boolean isRemote, String xpath) {
		// parse key to define type and label
		String[] type_label = MobileDeviceInitService.split2(key,"_");
		String fieldType = type_label[0];
		if (!isOptional)
			type_label[1] = type_label[1]+MobileDeviceInitUI.REQUIRED_MARKER;
				
		if (fieldType.equals("combobox")) {			//create the combobox
			if (isRemote) {
				//			String id  = new Long(now).toString();
				String idSubscription = new Long(System.currentTimeMillis()).toString();
				uiMap.put(key, this.createComboBox(value, key,type_label[1]));
				idSubscriptionMap.put(idSubscription, new Object[] {key,value/*uuid on belongsTo*/});
				cbx_callRemoteSubscription(xpath, idSubscription);				
			} else {
				//TODO (maybe)uiMap.put(key, this.createComboBox(value, key,type_label[1], new String[]{"1","2","3","4","5","6"}));

				String chocies[] = Utils.stringtoArray(value, ",");
				
				uiMap.put(key, this.createComboBox(key,type_label[1], chocies));
			}

		} else if (fieldType.equals("checkbox")) {
			// for all comma separated values
		} else { // must be string -> text
			uiMap.put(key, this.createTextField(value, key, type_label[1]));
		}
	}

	private void cbx_callRemoteSubscription(String xpath, String idSubscription) {		
		try {
			ContextRoutingService crs = ContextRoutingService.getInstance();
			crs.findSubscriptionDestination(xpath, idSubscription, this);
		} catch (Exception e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
			logger.error("The list of remote data requested could not be fetched, because of faulty xpath: '"+xpath+"'");
		}

	}/*
	private Object [] cbx_callRemoteQuery(String xpath) {
		ContextRoutingService crs = ContextRoutingService.getInstance();
		return crs.findQueryDestination(xpath);
	}*/

	/**
	 * @param text:
	 *            value to be preset
	 * @param name:
	 *            unique id of field
	 * @param labelText:
	 *            text of the associated label
	 * @return [label, textinput]
	 */
	private Object[] createTextField(String text, String name, String labelText) {
		Object textfield = create("textfield");
		setString(textfield, "name", name);
		setString(textfield, "text", text);
		Object label = this.createLabel(labelText);
		return new Object[] { label, textfield };
	}	
	/**
	 * returns the standard interaction buttons
	 * 
	 * @return [buttonNext, buttonCancle]
	 */
	private Object[] createStdButtons() {
		return new Object[] { this.createButton("Next >>", "next()"),
				this.createButton("Cancel", "cancel()") };
	}

	/**
	 * @param text
	 *            text on the button
	 * @param action
	 *            method to call on event
	 * @return the button as a object
	 */
	private Object createButton(String text, String action) {
		Object button = create("button");
		setString(button, "text", text);
		setMethod(button, "action", action, mainPanel, this);
		return button;
	}

	/**
	 * the button at the end of the user interaction sequence
	 * 
	 * @return [space, buttonDone]
	 */
	private Object[] createDoneButton() {
		return new Object[] { this.createLabel(""), // Dummy label
				this.createButton(DONE, "done()") }; 
	}

	private Object[] createNextButton() {
		return new Object[] { this.createLabel(""),
				this.createButton(NEXT, "next()")};
	}

	private Object[] createPrevButton() {
		return new Object[] { this.createLabel(""), // Dummy label
				this.createButton(PREV, "prev()")};
	}

	private boolean checkRequiredFields() {
		
		for (Iterator it = uiMap.values().iterator(); it.hasNext();) {
			Object[] components = (Object[]) it.next();
			if (components!= null && 
				components.length >= 2 && 
				isOfType(components[0], "label")) {
					Object label = components[0];
					String s = getString(label, "text");
					if (s.endsWith(MobileDeviceInitUI.REQUIRED_MARKER)) {
							String value = getValueFromField(components[1]);
							if (value.equals("")) {
								this.switchCurrentButton(this.createPrevButton());
								add(mainPanel, createLabel("Required:")); 
								add(mainPanel, createLabel(s,Color.RED));
								return false;		
							}
					}
			}
		}
		return true;
	}
	
	private void switchCurrentButton(Object[] newButton) {
		remove(buttonset[0]);
		remove(buttonset[1]);
		buttonset = newButton;
		add(mainPanel, buttonset[0]);
		add(mainPanel, buttonset[1]);
		repaint();
	}
	/** ************* button callback methods ************************** */
	/**
	 * send signal so caller can proceed
	 */
	public void done() {
		
		if (!checkRequiredFields())
			return;
		
		this.SIGNAL_RESULT();
		this.SIGNAL_UI();

		//unregister all callbacks
		if (idSubscriptionMap.size() > 0) {
			for (Iterator it = idSubscriptionMap.keySet().iterator(); it.hasNext(); ) {
				String idSubscription = (String) it.next();				
				try {
					ContextRoutingService.getInstance().removeSubscriptionDestination(idSubscription);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					logger.error("Could not remove subscription for remote data fetch");
				}
					
			}
		}
	}

	/**
	 * forward the uiIt and show next fields
	 */
	public void next() {
		this.addRemainingFields();
	}

	/**
	 * backward the uiIt
	 */
	public void prev() {
		this.startOverFields();
	}


	/**
	 * same as done
	 */
	public void cancel() {
		done();
	}
	private int getSelectedToObject(Object combobox, Object choice) {
		Object item[] = getItems(combobox);
		if (item == null || choice == null)
			return -1;
		
		
		
		for (int i=0; i < item.length; i++) {
			if (getString(item[i],"name").equals(getString(choice,"name")))
				return i;
		}
		return -1;
	}
	 /** creates a combobox choice
	 * @param combobox
	 * @param name unique identifier
	 * @param text human readable label
	 * @return
	 */
	private Object createChoice(Object combobox, String name, String text) {		
		Object choice = create("choice");
		setString(choice, "name", name);
		setString(choice, "text", text);
		add(combobox, choice,-1);
		return choice;
	}
	/**
	 * creates a combobox
	 * @param text not in use (but value to preset)
	 * @param name unique id of field 
	 * @param labelText 
	 * @return
	 */
	private Object[] createComboBox(String text, String name, String labelText) {
		Object label = this.createLabel(labelText);
		Object combobox = create("combobox");
		setString(combobox, "name", name);
		setBoolean(combobox, "editable", false);
		
		return new Object[] {label, combobox};
	}

	private Object[] createComboBox(String name, String labelText, String[] choices) {
		Object label = this.createLabel(labelText);
		Object combobox = create("combobox");
		setString(combobox, "name", name);
		setBoolean(combobox, "editable", false);
	
		for (int i=0; i < choices.length; i++) {
			createChoice(combobox, choices[i], choices[i]);
		}
		return new Object[] {label, combobox};
	}
	private Object createLabel(String text) {
		Object label = create("label");
		setString(label, "text", text);
		setChoice(label, "alignment", "right");
		return label;
	}

	private Object createLabel(String text, Color color) {
		Object label = create("label");
		setString(label, "text", text);
		setChoice(label, "alignment", "right");
		setColor(label,"foreground", color);
		return label;
	}

	public int getFieldsPerPage() {
		return fieldsPerPage;
	}
	/**
	 * synchronize opening of UI
	 */
	public void getUIAccess() {
		this.WAIT_UI();
	}
	public void setFieldsPerPage(int fieldsPerPage) {
		this.fieldsPerPage = fieldsPerPage;
	}

	public Map getUiMap() {
		return uiMap;
	}

	public String getValueFromField(Object component) {
		if(isOfType(component, "textfield"))
			return getString(component, "text");
		else if(isOfType(component, "combobox") && getSelectedItem(component) != null) {
			return getString(getSelectedItem(component),"name");
		}
		return "";
	}
	public void waitForResult() {
		this.WAIT_RESULT();
	}
	
	private boolean isOfType(Object component, String type) {
		Object [] theComponent;
		try {
			theComponent = (Object[])component;
		} catch (Exception e) { //especially casting exception
			return false;
		}
		if (theComponent.length < 2)
			return false;
				
		if (theComponent[1].equals(type))
			return true;
		
		return false;
	}
	/** ******************Synchronize the UI with business logic ****** */
	private synchronized void WAIT_UI() {
		while (tokenUI == 0) {
			try {
				wait();
			} catch (InterruptedException e) {

			}
		}
		tokenUI--;
		//System.out.println("GOTHERE_WAIT");
	}
	private synchronized void WAIT_RESULT() {
		while (tokenResult == 0) {
			try {
				wait();
			} catch (InterruptedException e) {

			}
		}
		tokenResult--;
		//System.out.println("GOTHERE_WAIT");
	}
	private synchronized void SIGNAL_RESULT() {
		tokenResult++;
		//System.out.println("GOTHERE_SIGNAL");
		notify();
	}

	private synchronized void SIGNAL_UI() {
		tokenUI++;
		//System.out.println("GOTHERE_SIGNAL");
		notify();
	}
	/**
	 * @param args
	 *            none
	 */
//	public static void main(String[] args) {
//
//		MobileDeviceInitUI UI = new MobileDeviceInitUI();
//		Iterator it;
//
//		
//		//also in a loop...
//		UI.generateFields("text_Firstname", "", true, false,"");
//		UI.generateFields("text_Lastname", "Simson", false, false,"");
//		UI.generateFields("text_Team", "", false, false,"");
//		
//		UI.setFieldsPerPage(2);
//		UI.addFieldsToUI();
//
//		new thinlet.FrameLauncher("Init Info", UI, 208, 256);
//		UI.waitForResult();
//		it = UI.getUiMap().keySet().iterator();
//		while (it.hasNext()) {
//			String key = (String) it.next();
//			Object[] o = (Object[]) UI.getUiMap().get(key);
//			System.out.println(key+"->"+UI.getValueFromField(o[1]));
//		}
//		
//		System.exit(0);
//	}
}
