/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.interfaces;

import at.ac.tuwien.vitalab.cosine.processing.notification.AContextQuery;

/**
 * Subscription provider interface for local subscriptions.
 * 
 * @author Lukasz Juszczyk
 *
 */
public interface ILocalSubscriptionProvider {

	/**
	 * Subscribes.
	 * 
	 * @param query Query object.
	 * @return Whether the object has been registered before.
	 * @throws Exception If errors occur while placing the query.
	 */
	public boolean subscribe(AContextQuery query) throws Exception;

	/**
	 * Unsubscribes.
	 * 
	 * @param query Query object.
	 * @throws Exception If errors occur while removing the query.
	 */
	public void unsubscribe(AContextQuery query) throws Exception;
	
}
