/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.interfaces;

import java.util.Vector;

import at.ac.tuwien.vitalab.cosine.ContextData;
import at.ac.tuwien.vitalab.cosine.processing.notification.SubscriptionClient;

/**
 * Notification Receiver Web service interface.
 * 
 * @author Lukasz Juszczyk
 *
 */
public interface INotificationReceiverWS {
	
	/**
	 * Notify about new context data which was subscribed to.
	 * 
	 * @param id Subscription ID.
	 * @param context Context data.
	 * @throws Exception If the ID or the context data is invalid.
	 */
	public void contextNotification(String id, ContextData context)
	throws Exception;
	
	/**
	 * Notify about a list of new context data which was subscribed to.
	 * 
	 * @param id Subscription ID.
	 * @param contextList Context data.
	 * @throws Exception If the ID or the context data is invalid.
	 */
	public void contextListNotification(String id, Vector contextList)
	throws Exception;
	
	/**
	 * Registers a subscription client which should be refreshed on incoming
	 * notifications.
	 * 
	 * @param id Subscription ID.
	 * @param sc Client.
	 */
	public void registerSubscriptionClient(String id, SubscriptionClient sc);
	
	/**
	 * Unregisters a subscription client.
	 * 
	 * @param id Subscription ID.
	 */
	public void unregisterSubscriptionClient(String id);
}
