/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.interfaces;

import edu.wustl.mobilab.sliver.soap.SOAPServer.InvocationContext;

/**
 * Subscription Provider Web Service Interface.
 * 
 * @author Lukasz Juszczyk
 * @author Florian Skopik
 *
 */
public interface ISubscriptionProviderWS {

	/**
	 * Subscribe to context data.
	 * 
	 * @param query Query string.
	 * @param id Subscription ID.
	 * @param context Invocation context to get WS-A header from.
	 * @return Whether the subscription has already been placed before.
	 * @throws Exception If the query is invalid.
	 */
	public boolean subscribe(String query, String id, InvocationContext context) 
	throws Exception;
	
	/**
	 * Unsubscribe from context data.
	 * 
	 * @param id Subscription ID.
	 * @throws Exception If the ID is invalid/unknown.
	 */
	public void unsubscribe(String id) throws Exception;
}
