/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.processing.notification;

import java.util.Vector;

import at.ac.tuwien.vitalab.cosine.ContextData;

/**
 * Abstract context query.
 * 
 * @author Lukasz Juszczyk
 *
 */
public abstract class AContextQuery {

	/**
	 * Stops processing this query.
	 *
	 */
	public abstract void stop();
	
	/**
	 * Returns the query string.
	 * 
	 * @return Query string.
	 */
	public abstract String getQueryString();
	
	/**
	 * Performs notification about changed context.
	 * 
	 * @param context Context data.
	 * 
	 * @throws InactiveSubscriptionException If the subscription has been 
	 * inactive for too long and should be removed.
	 */
	public abstract void contextQueryNotification(ContextData context)
	throws InactiveSubscriptionException;
	
	/**
	 * Performs notification about changed context.
	 * 
	 * @param contextList List of context data.
	 * 
	 * @throws InactiveSubscriptionException If the subscription has been 
	 * inactive for too long and should be removed.
	 */
	public abstract void contextQueryNotification(Vector contextList)
	throws InactiveSubscriptionException;
}
