/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.processing.storage;

import java.io.IOException;
import java.util.Vector;

import org.xmlpull.v1.XmlPullParserException;

import edu.wustl.mobilab.sliver.soap.MustUnderstandException;
import at.ac.tuwien.vitalab.middleware.discovery.InactiveServiceException;
import at.ac.tuwien.vitalab.middleware.discovery.MissingServiceAttributeException;
import at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo;

/**
 * Context query client.
 * 
 * @author Lukasz Juszczyk
 *
 */
public class QueryClient {

	/**
	 * Retrieves context data from a sensor.
	 * 
	 * @param sensor Sensor service to query.
	 * @param query Query string.
	 * 
	 * @return Resulting context data.
	 * 
	 * @throws IOException If IO errors occur.
	 * @throws XmlPullParserException If the XML messages cannot be 
	 * parsed properly. 
	 * @throws MustUnderstandException If the service cannot understand
	 * the request.
	 * @throws InactiveServiceException If the service is not active
	 * anymore.
	 * @throws MissingServiceAttributeException If some of the 
	 * necessary service attributes have not been published. 
	 */
	public static Vector getContextInfo(String query, ServiceInfo sensor)
		throws IOException, XmlPullParserException, MustUnderstandException, 
		InactiveServiceException, MissingServiceAttributeException {

		return (Vector)sensor.invokeOperation(
				"getContextInfo", new String[]{query},null);
	}

}
