/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.query;

import java.io.Reader;
import java.io.StringReader;

import at.ac.tuwien.vitalab.cosine.ContextData;

import ch.ethz.mxquery.exceptions.MXQueryException;
import ch.ethz.mxquery.model.iterators.CurrentBasedIterator;
import ch.ethz.mxquery.model.iterators.Iterator;
import ch.ethz.mxquery.model.iterators.XMLIterator;
import ch.ethz.mxquery.query.Context;
import ch.ethz.mxquery.util.tokens.Token;

/**
 * Context Document for XQuery.
 * 
 * @author Lukasz Juszczyk
 *
 */
public class ContextDoc extends CurrentBasedIterator {
	
	String contextString=null;
	
	/**
	 * Constructor.
	 * 
	 * @param context Context data.
	 */
	public ContextDoc(ContextData context) {

		super(null);
		
		// TODO: silly workaround via serialization into strings. CHANGE!!
		contextString=context.toString();
	}

	private ContextDoc() {
		super(null);
	}
	
	public Token next() throws MXQueryException {
		if (called == 0) {
			init();
			called++;
		}
		return current.next();
	}
	
	protected void init() throws MXQueryException {
		Reader reader=new StringReader(contextString);
		current = new XMLIterator(context, reader,true);		
	}

	public Iterator copy(Context context, Iterator[] subIters) throws MXQueryException {
		Iterator copy = new ContextDoc();
		((ContextDoc)copy).contextString=this.contextString;
		copy.setContext(context);
		copy.setSubIters(subIters);
		return copy;	
	}
}
