/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.query;

import java.io.StringReader;
import java.util.Vector;

import org.kxml2.io.KXmlParser;
import org.kxml2.kdom.Element;
import org.xmlpull.v1.XmlPullParser;

import at.ac.tuwien.vitalab.cosine.ContextData;
import ch.ethz.mxquery.functions.Function;
import ch.ethz.mxquery.model.QName;
import ch.ethz.mxquery.query.Context;
import ch.ethz.mxquery.query.PreparedStatement;
import ch.ethz.mxquery.query.impl.CompilerImpl;
import ch.ethz.mxquery.util.IteratorPrinter;

/**
 * XQuery wrapper.
 * 
 * @author Lukasz Juszczyk
 *
 */
public class XQuery {

	/**
	 * Returns all matching context.
	 * 
	 * @param context Context data.
	 * @param query Query string.
	 * 
	 * @return List of context data.
	 *
	 * @throws Exception If errors occur while processing the query.
	 */
	public static Vector returnMatches(ContextData context, String query) 
	throws Exception {
		Vector result=new Vector();

		query=query.trim();
                                		
		// return whole context
		if (query==null || query.length()==0 || query.equals("/*")) {
			result.add(context);
			return result;
		}
		
		// return all context items
		if ("TMP".equals(context.getName()) && 
				ContextData.NAMESPACE.equals(context.getNamespace()) &&
				query.equals("/TMP/*")) {
			for (int i=0;i<context.getChildCount();i++) {
				Object child=context.getChild(i);
				if (child instanceof Element) {
					result.add(child);
				}
			}
			return result;
		}
		
		if (query.startsWith("/")) {
			query="contextDataNamespace:contextData()"+query;
		}
		
		Context ctx=new Context();		
		Function function=new Function(new ContextDoc(context));
		ctx.addNamespace("contextDataNamespace", ContextData.NAMESPACE);
		ctx.addNamespace("", ContextData.NAMESPACE);
		
		QName qname=new QName("contextDataNamespace","contextData");
		
		ctx.addExternalFunction(qname, 0, true, function);
		
		CompilerImpl compiler=new CompilerImpl();
		PreparedStatement statement=compiler.compile(ctx, query);
		
		ch.ethz.mxquery.model.iterators.Iterator iter=statement.evaluate();
		String strResult = IteratorPrinter.eventsToXML(iter).toString();
		
		// no results?
		if (strResult.length()==0) {
			// return empty list
			return result;
		}
		
		XmlPullParser parser=new KXmlParser();
		parser.setInput(new StringReader(strResult));
		parser.setFeature(KXmlParser.FEATURE_PROCESS_NAMESPACES, true);
		
		parser.nextTag();
		
		while (parser.getEventType()!=XmlPullParser.END_DOCUMENT) {
			
			String name=parser.getName().intern();
			String namespace=parser.getNamespace();
			if (namespace!=null) {
				namespace=namespace.intern();
			}
			
			ContextData cd=new ContextData(name,namespace);

			cd.parse(parser);
			result.add(cd);
		}

		return result;
	}
}
