/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.rating;

import java.util.HashSet;

import at.ac.tuwien.vitalab.cosine.ContextData;
import at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo;

/**
 * Abstract class for rating extractions.
 * 
 * @author Lukasz Juszczyk
 *
 */
public abstract class ARating {

	private double max;
	private double min;
	private double weight;	
	
	private static HashSet modules=new HashSet();
	
	/**
	 * Constructor.
	 */
	public ARating() {
		modules.add(this);
	}

	/**
	 * Lists the currently registered rating modules.
	 * 
	 * @return Rating modules.
	 */
	public static HashSet getRatingModules() {
		return modules;
	}
	
	/**
	 * Returns the number of classes.
	 * 
	 * @return Number of classes.
	 */
	public static int numberOfRatings() {
		return modules.size();
	}
	
	/**
	 * Returns the rating value.
	 * 
	 * @param sensorService Source sensor service.
	 * @param context Context data to rate.
	 * @return Rating value.
	 * 
	 * @throws IllegalArgumentException If the rating cannot be extracted.
	 */
	public abstract double getRating(
			ServiceInfo sensorService, ContextData context)
	throws IllegalArgumentException;
	
	/**
	 * 
	 * @return
	 */
	public double getMaxRating() {
		return max;
	}

	/**
	 * 
	 * @return
	 */
	public double getMinRating() {
		return min;
	}

	/**
	 * 
	 * @param max
	 */
	public void setMaxRating(double max) {
		this.max = max;
	}

	/**
	 * 
	 * @param min
	 */
	public void setMinRating(double min) {
		this.min = min;
		
	}

	/**
	 * 
	 * @return
	 */
	public double getWeightRating() {
		return weight;
	}

	/**
	 * 
	 * @param weight
	 */
	public void setWeightRating(double weight) {
		this.weight = weight;
	}
	
}
