/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.rating;

import java.util.HashMap;

import at.ac.tuwien.vitalab.cosine.ContextData;
import at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo;

/**
 * Extracts the freshness of a context notification.
 * 
 * @author Lukasz Juszczyk
 *
 */
public class FreshnessRating extends ARating {

	private HashMap uptodatenessMap=new HashMap();
	
	/**
	 * Constructor.
	 */
	public FreshnessRating() {
		super();
	}

	/* (non-Javadoc)
	 * @see at.ac.tuwien.vitalab.cosine.rating.IRating#getRating(at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo, at.ac.tuwien.vitalab.cosine.ContextData)
	 */
	public double getRating(ServiceInfo sensorService, ContextData context) 
	throws IllegalArgumentException {

		if (context==null || sensorService==null) {
			throw new IllegalArgumentException(
					"Service and/or context are null.");
		}
		
		long cTs=context.getContextTimestamp();
		long sTs=context.getSendingTimestamp();
		
		if (cTs==-1 || sTs==-1) {
			throw new IllegalArgumentException("No timestamp.");
		}
		
		long tsDiff=sTs-cTs;
		
		long values[]=(long[])uptodatenessMap.get(sensorService);
		
		if (values==null) {
			values=new long[]{0,0};
			uptodatenessMap.put(sensorService, values);
		}
		
		long sum=values[0];
		long num=values[1];
		
		num++;
		sum+=tsDiff;
		
		return sum/num;
	}

}
