/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.cosine.webservice;

import java.io.*;

import org.ksoap2.serialization.Marshal;
import org.ksoap2.serialization.PropertyInfo;
import org.ksoap2.serialization.SoapSerializationEnvelope;
import org.kxml2.io.KXmlParser;
import org.kxml2.kdom.Element;
import org.xmlpull.v1.*;

import at.ac.tuwien.vitalab.cosine.ContextData;


/**
 * ContextData Marshaler.
 * 
* @author Lukasz Juszczyk
*
*/
public class MarshalContextData implements Marshal {

	/*
	 * (non-Javadoc)
	 * @see org.ksoap2.serialization.Marshal#readInstance(org.xmlpull.v1.XmlPullParser, java.lang.String, java.lang.String, org.ksoap2.serialization.PropertyInfo)
	 */
   public Object readInstance(XmlPullParser parser, String namespace, 
		   String name, PropertyInfo expected) 
   throws IOException, XmlPullParserException {
	   
	   parser.nextTag();

	   parser.setFeature(KXmlParser.FEATURE_PROCESS_NAMESPACES, true);

	   ContextData context=
		   new ContextData(parser.getName(),parser.getNamespace());

	   if (parser.getEventType()!=XmlPullParser.END_TAG) {
		   context.parse(parser);		   
	   } 

	   return context;
   }

   /*
    * (non-Javadoc)
    * @see org.ksoap2.serialization.Marshal#writeInstance(org.xmlpull.v1.XmlSerializer, java.lang.Object)
    */
   public void writeInstance(XmlSerializer writer, Object obj) 
   throws IOException {
	   Element elem=(Element)obj;
	   // serialize, nodes, text, and attributes
	   
	   elem.write(writer);
   }
   
   /*
    * (non-Javadoc)
    * @see org.ksoap2.serialization.Marshal#register(org.ksoap2.serialization.SoapSerializationEnvelope)
    */
   public void register(SoapSerializationEnvelope cm) {
       cm.addMapping(ContextData.NAMESPACE, ContextData.NAME, 
    		   ContextData.class, this);
   }

}
