/*
    This file is part of COSINE.

    COSINE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    COSINE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with COSINE. If not, see <http://www.gnu.org/licenses/>.
 */
package demo;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.Vector;

import at.ac.tuwien.vitalab.cosine.CMS;
import at.ac.tuwien.vitalab.cosine.ContextData;
import at.ac.tuwien.vitalab.cosine.processing.notification.AContextQuery;
import at.ac.tuwien.vitalab.cosine.processing.notification.InactiveSubscriptionException;
import at.ac.tuwien.vitalab.middleware.Middleware;

public class DemoSubscription {
	
	public static void main(String[] args) {

		try {

			Middleware.start();
			
			final CMS cms=CMS.getLocalCIMS();

			// run  until 'exit' is typed into stdin
			BufferedReader reader=
				new BufferedReader(new InputStreamReader(System.in));
			
			String inputLine=null;
			while (true) {

				System.out.print("Type in query string or 'exit':");
				
				if ((inputLine=reader.readLine())==null || 
					"exit".equals(inputLine.toLowerCase())) {
						break;
				}
				
				final String query=inputLine;
				
				try {
					AContextQuery q=new AContextQuery() {

						public void contextQueryNotification(ContextData context) throws InactiveSubscriptionException {
							
							System.out.println(query+" got context: "+context);
							
						}

						public void contextQueryNotification(Vector contextList) throws InactiveSubscriptionException {
							
							// pass to contextQueryNotification(ContextData)
							
							if (contextList==null) {
								return;
							}
							for (Iterator it=contextList.iterator();it.hasNext();) {
								try {
									this.contextQueryNotification((ContextData)it.next());
								} catch (Exception e) {
									e.printStackTrace();
								}
							}
						}

						public String getQueryString() {
							// give me all workers context
							return query;
						}

						public void stop() {
							// no cleanup necessary
						}
					};

					cms.subscribeLocally(q);
				} catch (Exception e) {
					System.err.println("Error: "+e.getMessage());
				}
			}
			
			cms.shutdown();
			
			Middleware.stop();
			
			System.exit(0);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
}

