/*
    This file is part of RESCUE.

    RESCUE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RESCUE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with RESCUE. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.middleware.soap;

import org.ksoap2.SoapEnvelope;
import org.ksoap2.SoapFault;
import org.ksoap2.serialization.SoapObject;

/**
 * SOAP response object. 
 * 
 * @author Lukasz Juszczyk
 *
 */
public class SoapResponse extends SoapObject {

	/**
	 * Constructor.
	 * 
	 * @param name Name of the service operation.
	 * @param namespace Namespace of the service.
	 * @param result Result object.
	 */
	public SoapResponse(String name, String namespace, Object result) {
		super(namespace, name+"Response");
		this.addProperty("return", result);
	}

	/**
	 * Constructor for void responses.
	 * 
	 * @param name Name of the service operation.
	 * @param namespace Namespace of the service.
	 */
	public SoapResponse(String name, String namespace) {
		super(namespace, name+"Response");
	}

	/**
	 * Retrieves the respons object.
	 * 
	 * @param response SOAP response.
	 * @return Respons eobject.
	 * @throws IllegalArgumentException If the passed object is not a 
	 * valid SOAP response.
	 */
	public static Object retrieveResultObject(Object response) 
	throws IllegalArgumentException {
		if (response instanceof SoapObject) {
			SoapObject responseObject=(SoapObject)response;
			if (responseObject.getName().endsWith("Response") &&
					responseObject.getPropertyCount()==1) {
				try {
					return responseObject.getProperty(0);							
				} catch (RuntimeException rte) {
				}
			}
		}
		return response;
//		throw new IllegalArgumentException("Not a soap response.");
	}

	/**
	 * Checks whether the response is a Fault and throws a SoapFault
	 * eventually.
	 * 
	 * @param response SOAP response.
	 * @throws at.ac.tuwien.vitalab.middleware.soap.SoapFault Fault thrown.
	 */
	public static void checkForFault(Object response) 
	throws at.ac.tuwien.vitalab.middleware.soap.SoapFault {
		
		// was a fault returned?
		if (response instanceof SoapObject) {
			SoapObject so=(SoapObject)response;
			if ("Fault".equals(so.getName()) && 
					SoapEnvelope.ENV.equals(so.getNamespace())) {
				SoapFault sf=new SoapFault();
				try {
					sf.faultstring=
						so.getProperty("faultstring").toString();
				} catch (NullPointerException npe) {
				}
				try {
					sf.faultcode=so.getProperty("faultcode").toString();
				} catch (NullPointerException npe) {
				}
				throw new at.ac.tuwien.vitalab.middleware.soap.SoapFault(sf);
			}
		}

	}
}