/*
    This file is part of RESCUE.

    RESCUE is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RESCUE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with RESCUE. If not, see <http://www.gnu.org/licenses/>.
 */
package demo;

import java.util.Hashtable;

import at.ac.tuwien.vitalab.middleware.Middleware;
import at.ac.tuwien.vitalab.middleware.discovery.AServiceQuery;
import at.ac.tuwien.vitalab.middleware.discovery.ICallbackReceiver;
import at.ac.tuwien.vitalab.middleware.discovery.IServiceSubscriber;
import at.ac.tuwien.vitalab.middleware.discovery.ServiceDB;
import at.ac.tuwien.vitalab.middleware.discovery.ServiceInfo;

/**
 * AsyncDemo.
 * 
 * @author Lukasz Juszczyk
 *
 */
public class AsyncDemo implements IServiceSubscriber, ICallbackReceiver {

	public static void main(String args[]) throws Exception {
		new AsyncDemo().run();
	}
 
	// create the query
	AServiceQuery q=new DemoQuery();
	Hashtable callbackIDs=new Hashtable();
	
	public void run() {
		
		try {
			
			Middleware.setConfigurationFileName("conf/middleware.conf");
			
			// start the middleware
			Middleware.start();
			
			// create the web service
			DemoService service=new DemoService();
			
			// deploy it
			Middleware.deployService(service);
			
			// get access to the service database
			ServiceDB db=Middleware.getServiceDatabase();

			System.out.println("1 = subscribe");
			System.out.println("2 = list services");
			System.out.println("x = exit");					

			// wait until key presses
			int c=0;
			boolean subscribed=false;
			while (c!='x') {
				
				c=System.in.read();			
				
				switch (c) {
				case '1' :
				case 'u' :
					if (!subscribed) {
						// subscribe to notifications
						subscribed=true;
						db.subscribe(q, this);
						System.out.println("subscribed");
					}
					break;
				case '2' :
				case 'i' :
					System.out.println("found services:");
					System.out.println(db.getServices());
				case 'x' :
					break;
				default:
					c=-1;
				}
				
			}
			
			// undeploy the service
			Middleware.undeployService(service);
			
			// stop the middleware
			Middleware.stop();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		System.exit(0);			
	}
	
	// this function is mandatory if the client implements IServiceSubscriber
	// it is invoked each time the queries trigger
	public void notification(AServiceQuery query, ServiceInfo info) {
		if (query==q) {
				try {
					
					Object id=info.invokeOperation(
							"getDate", new Object[]{}, this);
					callbackIDs.put(id, info);
					
				} catch (Exception e) {
					// invocation exceptions etc.
					e.printStackTrace();
				}
		}
	}


	// callback function for async WS call
	public void callback(String messageID, Object response) {
		String who=messageID;
		ServiceInfo info=(ServiceInfo)callbackIDs.remove(messageID);
		if (info!=null) {
			who=info.toString();
		}
		System.out.println(who+" says: "+response.toString());
	}
	
	// error callback function from async WS call
	public void error(String messageID, Exception e) {
		String who=messageID;
		ServiceInfo info=(ServiceInfo)callbackIDs.remove(messageID);
		if (info!=null) {
			who=info.toString();
		}
		System.err.println("error from "+who+": "+e.getMessage());
	}
	
	public class DemoQuery extends AServiceQuery {
		
		public boolean matches(ServiceInfo info) {

			try {
				if ("http://www.vitalab.tuwien.ac.at/demoservice".equals(info.getNamespace())
						&& "demoService".equals(info.getId())
						&& info.getOperationnames().contains("getDate")
						
						// remove the comment to avoid seeing our own services
//						&& !MiddlewareConfiguration.getHostUUID().equals(
//								info.getHostId())
								) {
					return true;
				}				
			} catch (Exception e) {
				return false;
			}
			return false;
		}
	}

}
