/**
 * 
 */
package test;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.MulticastSocket;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.StringTokenizer;

import at.ac.tuwien.vitalab.middleware.MiddlewareConfiguration;

/**
 * @author Lukasz Juszczyk
 *
 */
public class TestbedSimulator {

	private static int udpPort=MiddlewareConfiguration.getAdvertisementPort();
	private static MulticastSocket udpSocket=null;
	private static InetAddress mcastAddr=null;
	
	protected static double fieldSize=400/2;
	protected static double radioRange=50;
	protected static double reliability=2;
	
	protected static Random r=new Random();
	
//	protected static double[] dirs=new double[]{0.2.};
	protected static int counter=1;
	
	protected static TestbedFrame f;
	
	class Coords {
		
		public Coords(InetSocketAddress adr, String uuid) {
			this.adr=adr;
			this.uuid=uuid;
			direction=(counter++);
			f.addCircle(String.valueOf(adr.getPort()+1),(int)x, (int)y, (int)radioRange, (int)radioRange, counter-1);
		}
		
		InetSocketAddress adr=null;
		String uuid=null;
		double x=0.0;
		double y=0.0;
		double speed=10.0;
		double direction=0.0;
		
		public void proceed() {
			x+=Math.sin(direction)*speed;
			y+=Math.cos(direction)*speed;
			
			if (x>fieldSize || x<-fieldSize) {
				direction=(2*Math.PI)-direction;
			}

			if (y>fieldSize || y<-fieldSize) {
				direction=(Math.PI)-direction;
			}
			
			System.out.println(adr+" moved to "+this);
//			System.err.println(direction);
			f.moveCircle(String.valueOf(adr.getPort()+1), (int)x, (int)y);
		}
		
		public boolean inRange(Coords c) {
			double distance=Math.sqrt((c.x-x)*(c.x-x)+(c.y-y)*(c.y-y));
			if (distance<radioRange) {
				return r.nextDouble()<reliability;
			} else {
				return false;
			}
		}
		
		
		
		public int reachable(Coords c) {
			return reachable(c,new HashSet(),0);
		}
		
		public int reachable(Coords c, HashSet seen, int dep) {
			
			HashSet cs=new HashSet(ports.values());
			
			Iterator i=cs.iterator();
			
			int min=Integer.MAX_VALUE;
			
			while (i.hasNext()) {
				Coords t=(Coords)i.next();
				
				HashSet tseen=new HashSet(seen);
				
				if (this==t || seen.contains(t) || !inRange(t)) {
					continue;
				}

				if (inRange(c)) {
					return dep+1;
				}
				
				tseen.add(t);
				
				int res=t.reachable(c,tseen,dep+1);
				
				if (res!=-1 && res<min) {
					min=res;
				}
			}
			
			if (min==Integer.MAX_VALUE) {
				return -1;
			}
			
			return min;
		}
		
		public String toString() {
			return "<"+x+","+y+">";
		}
	}
	
	class Avail {
		long lastTime=System.currentTimeMillis();
		long time=0;
		long sum=0;
	}
	
	static Map ports=Collections.synchronizedMap(new HashMap());
	static Map avail=Collections.synchronizedMap(new HashMap());
	
	private static Avail getAvail(InetSocketAddress f, InetSocketAddress t ) {
		String key=f.toString()+" "+t.toString();
		Avail m=(Avail)avail.get(key);
		if (m==null) {
			m=new TestbedSimulator().new Avail();
			avail.put(key, m);
		}
		return m;
	}
	
	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {

		mcastAddr = InetAddress.getByName("239.255.255.253");
		
//		InetSocketAddress udpAddr=new InetSocketAddress(udpIP,udpPort); 
		
		udpSocket=new MulticastSocket(udpPort);
		udpSocket.joinGroup(mcastAddr);
		udpSocket.setTimeToLive(
				MiddlewareConfiguration.getAdvertisementTTL());

		new TestbedSimulator().new UDPReceiverThread().start();
		
		f = new TestbedFrame((int)fieldSize*2,(int)fieldSize*2);
		f.setVisible(true);
		new Thread() {

			/* (non-Javadoc)
			 * @see java.lang.Thread#run()
			 */
			public void run() {

				while (true) {
					
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
					}
					
					synchronized(ports) {
						Iterator it=ports.values().iterator();
						
						while (it.hasNext()) {
							Coords c=(Coords)it.next();
							
							c.proceed();
						}
					}
				}
			}
			
			
		}.start();
		
	}

	/**
	 * Thread which is responsible for receiving udp requests.
	 * 
	 * @author Lukasz Juszczyk
	 *
	 */
	private class UDPReceiverThread extends Thread {
		
		private boolean running=true;
		
		private void send(InetSocketAddress adr, String msg) {
			
			try {
				
				DatagramSocket socket=new DatagramSocket();
				DatagramPacket packetOut=new DatagramPacket(
						msg.getBytes(),msg.length(),adr);

//				System.out.println("SENDING '"+msg+"' to "+adr);
				
				socket.send(packetOut);
				
				if (socket!=null && !socket.isClosed()) {
					socket.close();						
				}
				
			} catch (Exception e) {
				e.printStackTrace();
			}
			
		}
		
		/*
		 * (non-Javadoc)
		 * @see java.lang.Thread#run()
		 */
		public void run() {
			
			
			while (running) {
				DatagramPacket packet;
				byte[] bytes = new byte[2000+128];
				packet = new DatagramPacket(bytes, bytes.length);
				String request=null;
				try {
					udpSocket.receive(packet);
					request=new String(packet.getData(),0,packet.getLength());

//					System.out.println("Got from "+packet.getAddress()+": "+
//							request);
					
					StringTokenizer tokens=new StringTokenizer(request);
					
					tokens.nextToken();
					String uuid=tokens.nextToken();
					String urlStr=tokens.nextToken();
					
					URL url=new URL(urlStr);
					
					InetSocketAddress advAddr=new InetSocketAddress(url.getHost(),url.getPort()-1); 

					if (!ports.containsKey(advAddr)) {
						ports.put(advAddr,new TestbedSimulator().new Coords(advAddr,uuid));						
					}

					Coords c=(Coords)ports.get(advAddr);
					
					Iterator i=ports.keySet().iterator();
					while (i.hasNext()) {
						InetSocketAddress a=(InetSocketAddress)i.next();
						
						if (a.equals(advAddr)) {
							continue;
						}
						
						Coords ac=(Coords)ports.get(a);
						
						int hops=0;
						if ((hops=ac.reachable(c))!=-1) {
							send(a,request);
							send(a,"hops "+uuid+" "+hops);
						} else {
//							System.err.print("NOT ");
						}
//						System.err.println("IN RANGE "+advAddr.getPort()+c+" and "
//								+a.getPort()+ac);
//						System.err.println(hops);
						
						Avail v=getAvail(advAddr,a);
						
						boolean isAvailable=hops!=-1;
						
						long dif=System.currentTimeMillis()-v.lastTime;
						if (isAvailable) {
							v.sum+=dif;
						}
						
						v.time+=dif;
						v.lastTime=System.currentTimeMillis();
						
						if (v.time!=0) {
							System.err.println("avail of '"+advAddr+"/"+a+"': "+
									100*((double)v.sum/(double)v.time)+"%");
							
						}
																
					}
					
					double counter=0;
					double sum=0;

					Iterator ai=avail.values().iterator();
					
					while (ai.hasNext()) {
						Avail a=(Avail)ai.next();
						
						counter+=a.time;
						sum+=a.sum;
						
					}

					System.err.println("general avail : "+
							100*(sum/counter)+"%");

				} catch (IOException e) {
					if (running) {
						System.err.println("Error while receiving UDP data: "+
								e.getMessage());						
					}
				} catch (Exception nsee) {
					if (running) {
						System.err.println("Received invalid UDP data '"+
								request+"'");						
					}
				}
			}
		}
	}
}
