/*
    This file is part of sensors developed for Vimoware.

    It is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    It is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with it. If not, see <http://www.gnu.org/licenses/>.
 */
package at.ac.tuwien.vitalab.sensors.workpad;

import at.ac.tuwien.vitalab.cosine.ContextData;
import at.ac.tuwien.vitalab.cosine.webservice.ASensorWebService;

/**
 * Battery Sensor Service.
 * 
 * @author Atif Manzoor
 * @author Lukasz Juszczyk
 *
 */
public class BatterySensorService extends ASensorWebService {

	private BatterySensor sensor = new BatterySensor();
	private BatterySensorThread thread = new BatterySensorThread();

	
	/**
	 * Constructor.
	 *
	 */
	public BatterySensorService() {
		
		super("Battery", ASensor.WORKPAD_SENSOR_NS, true, true);
	}

	/* (non-Javadoc)
	 * @see at.ac.tuwien.vitalab.middleware.AWebService#onDeploy()
	 */
	public void onDeploy() {
		super.onDeploy();
		thread.start();
	}


	/* (non-Javadoc)
	 * @see at.ac.tuwien.vitalab.middleware.AWebService#onUndeploy()
	 */
	public void onUndeploy() {
		thread.shutdown();
		super.onUndeploy();
	}


	/* (non-Javadoc)
	 * @see at.ac.tuwien.vitalab.middleware.AWebService#getConfigurationRessourceName()
	 */
	public String getConfigurationRessourceName() {
		return null;
	}

	/**
	 * Thread updating the battery status.
	 * 
	 * @author Lukasz Juszczyk
	 *
	 */
	private class BatterySensorThread extends Thread {

		boolean running;
		
		public BatterySensorThread() {
			running = true;
		}
		
		public void run() {
	
			while (running) {
				try {
					ContextData contextData = sensor.getContextDataXML();
					updateContext(contextData);			
				} catch (Exception e) {				
				}
				try {
					Thread.sleep(15000);
				} catch (InterruptedException e) {
					// ignore
				}
			}
		}
		
		public void shutdown() {
			running=false;
			interrupt();
		}
	}
	
}
